const kernelite = {};
const kJS = kernelite;

// Appel API
kernelite.api = function call( route, params = {} ) {
    return new Promise( (resolve,reject) => {
        fetch( URL+'api/'+route,{
            method : 'POST',
            headers: { 'Content-Type' : 'application/json;charset=UTF-8' },
            body : JSON.stringify( params ),       
        })
        .then( r => r.json() )
        .then( response => {
            if( !response?.success ) {
                kernelite.internalApiError(response?.errorMessage)
                kernelite.dismiss();
                reject();
            }
            kernelite.hideLoader();
            resolve(response)
        })
        .catch( err => {
            console.dir(err)
            kernelite.modal.error("Erreur serveur");
            kernelite.dismiss();
        })
    });
}

kernelite.internalApiError = ( (msg ) => {
    kernelite.hideLoader();
    kernelite.modal.fire({
        icon: 'error',
        title: 'Une erreur est survenue',
        html: msg == "" ?  "Aucun message d'erreur" : msg,
    })
});

// Modals - Documentation : https://sweetalert2.github.io/#examples
kernelite.modal = Swal;
kernelite.modal.success = ( (msg = {}) => {
    kernelite.hideLoader();
    kernelite.modal.fire({
        icon : 'success',
        text: msg
      })    
})
kernelite.modal.error = ( (msg = {}) => {
    kernelite.hideLoader();
    kernelite.modal.fire({
        icon : 'error',
        text: msg
      })    
})

kernelite.modal.confirm = ( params => {
    return new Promise( (resolve,reject) => {
        kernelite.modal.fire({
            title: params?.title ?? "Confirmation",
            html : params?.text ?? null,
            showDenyButton: true,
            target : 'body',
            confirmButtonText: params?.confirmButtonText ?? "Ok",
            denyButtonText: params?.denyButtonText ?? "Annuler",
          }).then( r => {
            if (r.isConfirmed) resolve();
            reject()
          })        
    })
})

// Notifications - Documentation @ https://github.com/caroso1222/notyf
kernelite.notyf = new Notyf({
    duration: 3000,
    types: [
        {
          type: 'warning',
          background: 'linear-gradient(45deg, rgb(239, 253, 33), rgb(255, 0, 0))',
          color : 'white',
          icon: {
            className: 'fa-solid fa-gear rotate',
            tagName: 'i',
            text: '',
            color : 'white'
          }
        }
      ]    
});
kernelite.success = ( (msg,dismissAll = true)  => {
    if( dismissAll ) kernelite.notyf.dismissAll();
    kernelite.notyf.success(msg) 
    return this;
});
kernelite.error = ( (msg,dismissAll = true) => {
    if( dismissAll ) kernelite.notyf.dismissAll();
    kernelite.notyf.error(msg) 
    return this;
});
kernelite.dismiss = () => kernelite.notyf.dismissAll();
kernelite.progress = ( (msg,dismissible = false) => {
    kernelite.notyf.open({
        type: 'warning',
        message: msg,
        dismissible : dismissible,
        duration : 0
    });
    return this;
})




// Dates
kernelite.date = {
    getTimestamp : ( str = false ) => str ? Date.parse(str) : Date.now(),
    get : ( format = false, full = false, $value = false ) => {
        let date = $value ? new Date($value) : new Date(),
            year = date.getFullYear(),
            month = date.getMonth() + 1,
            day = date.getDate(),
            hours = date.getHours(),
            min = date.getMinutes(),
            sec = date.getSeconds();
        if( day < 10 ) day = "0"+day
        if( month < 10 ) month = "0"+month
        if( hours < 10 ) hours = "0"+hours
        if( min < 10 ) min = "0"+min
        if( sec < 10 ) sec = "0"+sec
        if( !full ) 
            return format ? [day,month,year].join('/') : [year,month,day].join('-');
        if( format )
            return [day,month,year].join('/') + ' ' + hours + "h" + min + "min" + sec;
        return [year,month,day].join('-') + ' ' + [hours,min,sec].join(':');        
    }
}


// Events
kernelite.events = {
    datepicker : () => {
        $(".kernel-datepicker").each(function(){
            $(this).datepicker();
        })
    }
}
kernelite.fireEvent = (eventName) => {
    if( kernelite.events.hasOwnProperty(eventName) )
        kernelite.events[eventName]();
    else
        kernelite.error("Event introuvable : "+eventName)
}

// Login
kernelite.login = ( datas => {
    return new Promise( (resolve,reject) => {
        fetch( URL,{
            method : 'POST',
            body : JSON.stringify(datas),       
        })
        .then( r => r.json() )
        .then( response => {
            if( !response?.success ) return kernelite.modal.error(response?.errorMessage);
            if( !response?.login ) reject();
            if( response?.login === true )resolve();
        })
        .catch( err => {
            console.dir(err)
            reject();
        })
    })
})


// Checks
kernelite.isNull = ( value => {
    if( typeof value == "string" && value.trim() == "" ) return true;
    else if( typeof value == "integer" && parseInt(value) == 0 ) return true;
    else if( typeof value == "float" && parseFloat(value) == 0 ) return true;
    else if( value === undefined || typeof value == "undefined" || value === null ) return true;
    return false;
});
kernelite.length = ( o => {
    return (typeof o == "object"  ? Object.keys(o).length : o.length );
})


// Tools
kernelite.reload = () => window.location.reload()
kernelite.refresh = () => kernelite.reload()
kernelite.getURL = () => window.location.href;
kernelite.inURL = (needle) => kernelite.getURL().indexOf(needle) > 0

// Files
kernelite.upload = function(props) {
    return new Promise(resolve => {
        kernelite.generateInputFile(props).then( o => {
            let fd = new FormData();     
            for( let i in o.files ) {
                if( props?.size && o.files[i].size > props.size )
                    return kernelite.modal.error("Le fichier est trop lourd<br/>Limite de poids : "+kernelite.humanFileSize(props.size));
                fd.append('kerneliteUpload[]', o.files[i]);
            }
            fetch(URL+'kernelite/asyncFileUpload',{
                method : 'POST',
                body : fd
            })
            .then( r => r.json() )
            .then( response => {
                if( !response?.success ) {
                    kernelite.internalApiError(response?.errorMessage)
                    return false;
                }    
                resolve(response);  
            })
        }); 
    })
};
kernelite.generateInputFile = (props) => {
    return new Promise( resolve => {
        let name = '_kerneliteTmpFile';
        if( $("#"+name).length ) $("#"+name).remove();
        $('body').append('<input type="file" id="'+name+'" style="display:none"/>');       
        let el = $("#"+name);
        if( props?.multiple ) el.prop('multiple',true);
        if( props?.accept ) el.attr('accept', props.accept );
        el.trigger('click').on('change', function() {
            resolve({
                obj : $(this)[0],
                files : $(this)[0].files
            });
        });
    })
}
kernelite.humanFileSize = (bytes, si=false, dp=1) => {
    const thresh = si ? 1000 : 1024;
    if (Math.abs(bytes) < thresh) 
      return bytes + ' B';
    const units = si 
      ? ['kB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'] 
      : ['KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB'];
    let u = -1;
    const r = 10**dp;

    do {
      bytes /= thresh;
      ++u;
    } while (Math.round(Math.abs(bytes) * r) / r >= thresh && u < units.length - 1);
    return bytes.toFixed(dp) + ' ' + units[u];
}
kernelite.removeTmp = ( tmp => {
    return new Promise( (resolve,reject) => {
        fetch(URL+'kernelite/removeTmpDir',{
            method : 'POST',
            body : JSON.stringify({ tmpDir : tmp })
        })
        .then( r => r.json() )
        .then( response => {
            if( !response?.success ) {
                kernelite.internalApiError(response?.errorMessage)
                reject();
            }    
            resolve(response);  
        })    
    })
})


kernelite.formToJson = ( idForm ) => {
    let f = $("#"+idForm), obj = {};
    if( f.length == 0 ) 
        kernelite.error("Impossible de trouver le formulaire #"+idForm);
    else {
        let Arr = f.serializeArray();
        for( let i in Arr ) obj[Arr[i].name] = Arr[i].value
    }    
    return obj;
}


// Loader
kernelite.loaderName = 'kernelite-loader-template';
kernelite.showLoader = () => {
    let k = $('body').find('#'+kernelite.loaderName);
    if( k.length && !k.is(':visible') ) k.show();
}
kernelite.hideLoader = () => {
    let k = $('body').find('#'+kernelite.loaderName);
    if( k.length && k.is(':visible') ) k.hide();
}


// Connection
kernelite.isConnected = async () => {
    const state = await fetch(URL+'kernelite/isConnected')
    .then( r => r.json() )
    .then( response => { return response.connected }) 
    return state;
}

// Navigation
kernelite.navigate = ( path => {
    return $(location).attr('href', URL+path );
})
kernelite.nav = (path) => kernelite.navigate(path)


// Params - with Cookies
kernelite.paramsKeyString  = "_kernelite_settings_";
kernelite.defaultExpireParam = 30; // 30 jours par defaut
kernelite.paramsObject = {};
kernelite.params = {
    init : () => {
        let object = Cookies.get(kernelite.paramsKeyString)
        if( !kernelite.isNull(object) ) {
            kernelite.paramsObject = JSON.parse( object );
        }
    },
    set : ( key, value ) => {
        kernelite.paramsObject[key] = value;
        kernelite.params.save();    
    },
    get : (key) => {
        return kernelite.paramsObject[key] ?? null;
    },
    reset : () => {
        kernelite.paramsObject = {};
        kernelite.params.save();        
    },
    save : () => {
        let date = new Date();
        Cookies.set( 
            kernelite.paramsKeyString, 
            JSON.stringify( kernelite.paramsObject ),
            {
                url : URL,
                expires : kernelite.defaultExpireParam
            }
        );
    }
}



// Tooltips 
kernelite.tooltips = () => {
    var tooltipTriggerList = [].slice.call(document.querySelectorAll('[data-bs-toggle="tooltip"]'))
    var tooltipList = tooltipTriggerList.map(function (tooltipTriggerEl) {
      return new bootstrap.Tooltip(tooltipTriggerEl,{ html : true })
    })    
}


// Layer
kernelite.layer = ( params ) => {
    kernelite.modal.fire({
        template : '#kernelite-layer-template',
        html : params?.html,
        width : params?.width ?? '90%',
        confirmButtonText : params?.confirmButtonText ?? "Fermer",
        customClass: {
            container : params?.class ?? 'kernelite-layer-class'
        }
    })
}